@@ -68,7 +68,7 @@ static const char *zend_vm_opcodes_names[203] = {
 	"ZEND_JMP",
 	"ZEND_JMPZ",
 	"ZEND_JMPNZ",
-	"ZEND_JMPZNZ",
+	NULL,
 	"ZEND_JMPZ_EX",
 	"ZEND_JMPNZ_EX",
 	"ZEND_CASE",
@@ -274,7 +274,7 @@ static uint32_t zend_vm_opcodes_flags[203] = {
 	0x00000020,
 	0x00002007,
 	0x00002007,
-	0x03002007,
+	0x00000000,
 	0x00002007,
 	0x00002007,
 	0x00000705,
