@@ -61,7 +61,7 @@ static void observer_set_user_opcode_handler(const char *opcode_names, user_opco
 
 static void observer_show_opcode(zend_execute_data *execute_data)
 {
-	if (!ZT_G(observer_show_opcode)) {
+	if (!ZT_G(observer_show_opcode) || !ZEND_USER_CODE(EX(func)->type)) {
 		return;
 	}
 	php_printf("%*s<!-- opcode: '%s' -->\n", 2 * ZT_G(observer_nesting_depth), "", zend_get_opcode_name(EX(opline)->opcode));
@@ -184,7 +184,7 @@ static zend_observer_fcall_handlers observer_fcall_init(zend_execute_data *execu
 	} else if (fbc->common.function_name) {
 		if (ZT_G(observer_observe_functions)) {
 			return (zend_observer_fcall_handlers){observer_begin, observer_end};
-		} else if (ZT_G(observer_observe_function_names) && zend_hash_exists(ZT_G(observer_observe_function_names), fbc->common.function_name)) {
+		} else if (zend_hash_exists(ZT_G(observer_observe_function_names), fbc->common.function_name)) {
 			return (zend_observer_fcall_handlers){observer_begin, observer_end};
 		}
 	} else {
@@ -256,6 +256,18 @@ static void fiber_suspend_observer(zend_fiber_context *from, zend_fiber_context
 	}
 }
 
+void declared_function_observer(zend_op_array *op_array, zend_string *name) {
+	if (ZT_G(observer_observe_declaring)) {
+		php_printf("%*s<!-- declared function '%s' -->\n", 2 * ZT_G(observer_nesting_depth), "", ZSTR_VAL(name));
+	}
+}
+
+void declared_class_observer(zend_class_entry *ce, zend_string *name) {
+	if (ZT_G(observer_observe_declaring)) {
+		php_printf("%*s<!-- declared class '%s' -->\n", 2 * ZT_G(observer_nesting_depth), "", ZSTR_VAL(name));
+	}
+}
+
 static void (*zend_test_prev_execute_internal)(zend_execute_data *execute_data, zval *return_value);
 static void zend_test_execute_internal(zend_execute_data *execute_data, zval *return_value) {
 	zend_function *fbc = execute_data->func;
@@ -280,19 +292,38 @@ static void zend_test_execute_internal(zend_execute_data *execute_data, zval *re
 static ZEND_INI_MH(zend_test_observer_OnUpdateCommaList)
 {
 	zend_array **p = (zend_array **) ZEND_INI_GET_ADDR();
-	if (*p) {
-		zend_hash_release(*p);
+	zend_string *funcname;
+	zend_function *func;
+	if (stage != PHP_INI_STAGE_STARTUP && stage != PHP_INI_STAGE_ACTIVATE && stage != PHP_INI_STAGE_DEACTIVATE && stage != PHP_INI_STAGE_SHUTDOWN) {
+		ZEND_HASH_FOREACH_STR_KEY(*p, funcname) {
+			if ((func = zend_hash_find_ptr(EG(function_table), funcname))) {
+				zend_observer_remove_begin_handler(func, observer_begin);
+				zend_observer_remove_end_handler(func, observer_end);
+			}
+		} ZEND_HASH_FOREACH_END();
 	}
-	*p = NULL;
+	zend_hash_clean(*p);
 	if (new_value && ZSTR_LEN(new_value)) {
-		*p = malloc(sizeof(HashTable));
-		_zend_hash_init(*p, 8, ZVAL_PTR_DTOR, 1);
 		const char *start = ZSTR_VAL(new_value), *ptr;
 		while ((ptr = strchr(start, ','))) {
-			zend_hash_str_add_empty_element(*p, start, ptr - start);
+			zend_string *str = zend_string_init(start, ptr - start, 1);
+			GC_MAKE_PERSISTENT_LOCAL(str);
+			zend_hash_add_empty_element(*p, str);
+			zend_string_release(str);
 			start = ptr + 1;
 		}
-		zend_hash_str_add_empty_element(*p, start, ZSTR_VAL(new_value) + ZSTR_LEN(new_value) - start);
+		zend_string *str = zend_string_init(start, ZSTR_VAL(new_value) + ZSTR_LEN(new_value) - start, 1);
+		GC_MAKE_PERSISTENT_LOCAL(str);
+		zend_hash_add_empty_element(*p, str);
+		zend_string_release(str);
+		if (stage != PHP_INI_STAGE_STARTUP && stage != PHP_INI_STAGE_ACTIVATE && stage != PHP_INI_STAGE_DEACTIVATE && stage != PHP_INI_STAGE_SHUTDOWN) {
+			ZEND_HASH_FOREACH_STR_KEY(*p, funcname) {
+				if ((func = zend_hash_find_ptr(EG(function_table), funcname))) {
+					zend_observer_add_begin_handler(func, observer_begin);
+					zend_observer_add_end_handler(func, observer_end);
+				}
+			} ZEND_HASH_FOREACH_END();
+		}
 	}
 	return SUCCESS;
 }
@@ -303,7 +334,8 @@ PHP_INI_BEGIN()
 	STD_PHP_INI_BOOLEAN("zend_test.observer.observe_all", "0", PHP_INI_SYSTEM, OnUpdateBool, observer_observe_all, zend_zend_test_globals, zend_test_globals)
 	STD_PHP_INI_BOOLEAN("zend_test.observer.observe_includes", "0", PHP_INI_SYSTEM, OnUpdateBool, observer_observe_includes, zend_zend_test_globals, zend_test_globals)
 	STD_PHP_INI_BOOLEAN("zend_test.observer.observe_functions", "0", PHP_INI_SYSTEM, OnUpdateBool, observer_observe_functions, zend_zend_test_globals, zend_test_globals)
-	STD_PHP_INI_ENTRY("zend_test.observer.observe_function_names", "", PHP_INI_SYSTEM, zend_test_observer_OnUpdateCommaList, observer_observe_function_names, zend_zend_test_globals, zend_test_globals)
+	STD_PHP_INI_BOOLEAN("zend_test.observer.observe_declaring", "0", PHP_INI_SYSTEM, OnUpdateBool, observer_observe_declaring, zend_zend_test_globals, zend_test_globals)
+	STD_PHP_INI_ENTRY("zend_test.observer.observe_function_names", "", PHP_INI_ALL, zend_test_observer_OnUpdateCommaList, observer_observe_function_names, zend_zend_test_globals, zend_test_globals)
 	STD_PHP_INI_BOOLEAN("zend_test.observer.show_return_type", "0", PHP_INI_SYSTEM, OnUpdateBool, observer_show_return_type, zend_zend_test_globals, zend_test_globals)
 	STD_PHP_INI_BOOLEAN("zend_test.observer.show_return_value", "0", PHP_INI_SYSTEM, OnUpdateBool, observer_show_return_value, zend_zend_test_globals, zend_test_globals)
 	STD_PHP_INI_BOOLEAN("zend_test.observer.show_init_backtrace", "0", PHP_INI_SYSTEM, OnUpdateBool, observer_show_init_backtrace, zend_zend_test_globals, zend_test_globals)
@@ -337,6 +369,9 @@ void zend_test_observer_init(INIT_FUNC_ARGS)
 		zend_observer_fiber_switch_register(fiber_enter_observer);
 		zend_observer_fiber_switch_register(fiber_suspend_observer);
 		zend_observer_fiber_destroy_register(fiber_destroy_observer);
+
+		zend_observer_function_declared_register(declared_function_observer);
+		zend_observer_class_linked_register(declared_class_observer);
 	}
 
 	if (ZT_G(observer_execute_internal)) {
@@ -350,8 +385,14 @@ void zend_test_observer_shutdown(SHUTDOWN_FUNC_ARGS)
 	if (type != MODULE_TEMPORARY) {
 		UNREGISTER_INI_ENTRIES();
 	}
+}
 
-	if (ZT_G(observer_observe_function_names)) {
-		zend_hash_release(ZT_G(observer_observe_function_names));
-	}
+void zend_test_observer_ginit(zend_zend_test_globals *zend_test_globals) {
+	zend_test_globals->observer_observe_function_names = malloc(sizeof(HashTable));
+	_zend_hash_init(zend_test_globals->observer_observe_function_names, 8, ZVAL_PTR_DTOR, 1);
+	GC_MAKE_PERSISTENT_LOCAL(zend_test_globals->observer_observe_function_names);
+}
+
+void zend_test_observer_gshutdown(zend_zend_test_globals *zend_test_globals) {
+	zend_hash_release(zend_test_globals->observer_observe_function_names);
 }
